﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Reflection;
using Newtonsoft.Json;

namespace ActiveXLibrary
{
    [Guid("C536CC09-EBB6-47CE-94E3-D9C4290120E3")]
    public class ActiveXRecord : UserControl, IObjectSafety
    {
        #region IObjectSafety 成员

        private const string _IID_IDispatch = "{00020400-0000-0000-C000-000000000046}";
        private const string _IID_IDispatchEx = "{a6ef9860-c720-11d0-9337-00a0c90dcaa9}";
        private const string _IID_IPersistStorage = "{0000010A-0000-0000-C000-000000000046}";
        private const string _IID_IPersistStream = "{00000109-0000-0000-C000-000000000046}";
        private const string _IID_IPersistPropertyBag = "{37D84F60-42CB-11CE-8135-00AA004BB851}";

        private const int INTERFACESAFE_FOR_UNTRUSTED_CALLER = 0x00000001;
        private const int INTERFACESAFE_FOR_UNTRUSTED_DATA = 0x00000002;
        private const int S_OK = 0;
        private const int E_FAIL = unchecked((int)0x80004005);
        private const int E_NOINTERFACE = unchecked((int)0x80004002);

        private bool _fSafeForScripting = true;
        private bool _fSafeForInitializing = true;


        public int GetInterfaceSafetyOptions(ref Guid riid, ref int pdwSupportedOptions, ref int pdwEnabledOptions)
        {
            int Rslt = E_FAIL;

            string strGUID = riid.ToString("B");
            pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA;
            switch (strGUID)
            {
                case _IID_IDispatch:
                case _IID_IDispatchEx:
                    Rslt = S_OK;
                    pdwEnabledOptions = 0;
                    if (_fSafeForScripting == true)
                        pdwEnabledOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER;
                    break;
                case _IID_IPersistStorage:
                case _IID_IPersistStream:
                case _IID_IPersistPropertyBag:
                    Rslt = S_OK;
                    pdwEnabledOptions = 0;
                    if (_fSafeForInitializing == true)
                        pdwEnabledOptions = INTERFACESAFE_FOR_UNTRUSTED_DATA;
                    break;
                default:
                    Rslt = E_NOINTERFACE;
                    break;
            }

            return Rslt;
        }

        public int SetInterfaceSafetyOptions(ref Guid riid, int dwOptionSetMask, int dwEnabledOptions)
        {
            int Rslt = E_FAIL;

            string strGUID = riid.ToString("B");
            switch (strGUID)
            {
                case _IID_IDispatch:
                case _IID_IDispatchEx:
                    if (((dwEnabledOptions & dwOptionSetMask) == INTERFACESAFE_FOR_UNTRUSTED_CALLER) &&
                            (_fSafeForScripting == true))
                        Rslt = S_OK;
                    break;
                case _IID_IPersistStorage:
                case _IID_IPersistStream:
                case _IID_IPersistPropertyBag:
                    if (((dwEnabledOptions & dwOptionSetMask) == INTERFACESAFE_FOR_UNTRUSTED_DATA) &&
                            (_fSafeForInitializing == true))
                        Rslt = S_OK;
                    break;
                default:
                    Rslt = E_NOINTERFACE;
                    break;
            }

            return Rslt;
        }

        #endregion

        #region

        /// <summary>
        /// 录音核心类
        /// </summary>
        private RecordCore recordCore = null;

        private static string BasePath = Path.Combine(System.Environment.GetFolderPath(Environment.SpecialFolder.Personal), @"RecordTmp\");

        public ActiveXRecord()
        {
            this.recordCore = new RecordCore();
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
        }

        #endregion

        #region JS调用C#方法

        /// <summary>
        /// 开始录音
        /// </summary>
        /// <returns></returns>
        public string StartRecord()
        {
            RecordInfo ri = new RecordInfo() { code = 100, msg = "" };
            try
            {
                this.recordCore.StartRecord();
                return JsonConvert.SerializeObject(ri);
            }
            catch (Exception ex)
            {
                ri.code = 600;
                ri.msg = ex.Message;
                return JsonConvert.SerializeObject(ri);
            }
        }

        /// <summary>
        /// 暂停录音
        /// </summary>
        /// <returns></returns>
        public string PauseRecord()
        {
            RecordInfo ri = new RecordInfo() { code = 100, msg = "" };
            try
            {
                if (this.recordCore.RecordStatus == RecordStatus.Record)
                {
                    this.recordCore.PauseRecord();
                }
                return JsonConvert.SerializeObject(ri);
            }
            catch (Exception ex)
            {
                ri.code = 600;
                ri.msg = ex.Message;
                return JsonConvert.SerializeObject(ri);
            }
        }

        /// <summary>
        /// 继续录音
        /// </summary>
        /// <returns></returns>
        public string ResumeRecord()
        {
            RecordInfo ri = new RecordInfo() { code = 100, msg = "" };
            try
            {
                if (this.recordCore.RecordStatus == RecordStatus.Pause)
                {
                    this.recordCore.ResumeRecord();
                }
                return JsonConvert.SerializeObject(ri);
            }
            catch (Exception ex)
            {
                ri.code = 600;
                ri.msg = ex.Message;
                return JsonConvert.SerializeObject(ri);
            }
        }

        /// <summary>
        /// 保存录音
        /// </summary>
        /// <returns></returns>
        public string SaveRecord()
        {
            RecordInfo ri = new RecordInfo() { code = 100, msg = "" };
            try
            {
                if (!Directory.Exists(BasePath))
                {
                    Directory.CreateDirectory(BasePath);
                }
                string filename = String.Format("{0}{1}.wav", BasePath, GetGuid());

                this.recordCore.SaveRecord(filename);
                ri.msg = filename;
                return JsonConvert.SerializeObject(ri);
            }
            catch (Exception ex)
            {
                ri.code = 600;
                ri.msg = ex.Message;
                return JsonConvert.SerializeObject(ri);
            }
        }

        /// <summary>
        /// 关闭录音
        /// </summary>
        /// <returns></returns>
        public string CloseRecord()
        {
            RecordInfo ri = new RecordInfo() { code = 100, msg = "" };
            try
            {
                if (this.recordCore.RecordStatus != RecordStatus.Close)
                {
                    this.recordCore.CloseRecord();
                }
                return JsonConvert.SerializeObject(ri);
            }
            catch (Exception ex)
            {
                ri.code = 600;
                ri.msg = ex.Message;
                return JsonConvert.SerializeObject(ri);
            }
        }

        /// <summary>
        /// 获取录音状态
        /// </summary>
        /// <returns></returns>
        public string GetRecordStatus()
        {
            RecordInfo ri = new RecordInfo() { code = 100, msg = "" };
            try
            {
                ri.msg = this.recordCore.RecordStatus.ToString();
                return JsonConvert.SerializeObject(ri);
            }
            catch (Exception ex)
            {
                ri.code = 600;
                ri.msg = ex.Message;
                return JsonConvert.SerializeObject(ri);
            }
        }

        /// <summary>
        /// 清除录音
        /// </summary>
        /// <returns></returns>
        public string ClearRecord(string filePath)
        {
            RecordInfo ri = new RecordInfo() { code = 100, msg = "" };
            try
            {
                if (File.Exists(filePath))
                {
                    File.Delete(filePath);
                }
                return JsonConvert.SerializeObject(ri);
            }
            catch (Exception ex)
            {
                ri.code = 600;
                ri.msg = ex.Message;
                return JsonConvert.SerializeObject(ri);
            }
        }


        #endregion

        /// <summary>
        /// 生成唯一编号
        /// </summary>
        /// <returns></returns>
        private string GetGuid()
        {
            string result = Guid.NewGuid().ToString().ToLower();
            result = result.Replace("0", "a");
            result = result.Replace("1", "b");
            result = result.Replace("2", "c");
            result = result.Replace("3", "d");
            result = result.Replace("4", "e");
            result = result.Replace("5", "f");
            result = result.Replace("6", "g");
            result = result.Replace("7", "h");
            result = result.Replace("8", "i");
            result = result.Replace("9", "j");
            return result;
        }

        /// <summary>
        /// 返回信息
        /// </summary>
        public class RecordInfo
        {
            /// <summary>
            /// 信息编号
            /// </summary>
            public int code { get; set; }
            /// <summary>
            /// 信息
            /// </summary>
            public string msg { get; set; }
        }
    }
}
